@@ -21,6 +21,7 @@ from xonsh.platform import (
     ON_NETBSD,
     ON_OPENBSD,
     ON_DRAGONFLY,
+    ON_POSIX,
 )
 from xonsh.tools import (
     XonshError,
@@ -33,7 +34,6 @@ from xonsh.tools import (
     unthreadable,
     print_color,
 )
-from xonsh.replay import replay_main
 from xonsh.timings import timeit_alias
 from xonsh.xontribs import xontribs_main
 from xonsh.ast import isexpression
@@ -41,6 +41,10 @@ from xonsh.ast import isexpression
 import xonsh.completers._aliases as xca
 import xonsh.history.main as xhm
 import xonsh.xoreutils.which as xxw
+from xonsh.xoreutils.umask import umask
+
+if ON_POSIX:
+    from xonsh.xoreutils.ulimit import ulimit
 
 
 @lazyobject
@@ -555,7 +559,7 @@ def source_alias(args, stdin=None):
                     print("source: {}: No such file".format(fname), file=sys.stderr)
                 if i == 0:
                     raise RuntimeError(
-                        "must source at least one file, " + fname + "does not exist."
+                        "must source at least one file, " + fname + " does not exist."
                     )
                 break
         _, fext = os.path.splitext(fpath)
@@ -609,7 +613,7 @@ def source_cmd(args, stdin=None):
 
 
 def xexec(args, stdin=None):
-    """exec [-h|--help] command [args...]
+    """exec [-h|--help] [-cl] [-a name] command [args...]
 
     exec (also aliased as xexec) uses the os.execvpe() function to
     replace the xonsh process with the specified program. This provides
@@ -619,6 +623,13 @@ def xexec(args, stdin=None):
         bash $
 
     The '-h' and '--help' options print this message and exit.
+    If the '-l' option is supplied, the shell places a dash at the
+    beginning of the zeroth argument passed to command to simulate login
+    shell.
+    The '-c' option causes command to be executed with an empty environment.
+    If '-a' is supplied, the shell passes name as the zeroth argument
+    to the executed command.
+
 
     Notes
     -----
@@ -632,18 +643,40 @@ def xexec(args, stdin=None):
     """
     if len(args) == 0:
         return (None, "xonsh: exec: no args specified\n", 1)
-    elif args[0] == "-h" or args[0] == "--help":
+
+    parser = argparse.ArgumentParser(add_help=False)
+    parser.add_argument("-h", "--help", action="store_true")
+    parser.add_argument("-l", dest="login", action="store_true")
+    parser.add_argument("-c", dest="clean", action="store_true")
+    parser.add_argument("-a", dest="name", nargs="?")
+    parser.add_argument("command", nargs=argparse.REMAINDER)
+    args = parser.parse_args(args)
+
+    if args.help:
         return inspect.getdoc(xexec)
-    else:
+
+    if len(args.command) == 0:
+        return (None, "xonsh: exec: no command specified\n", 1)
+
+    command = args.command[0]
+    if args.name is not None:
+        args.command[0] = args.name
+    if args.login:
+        args.command[0] = "-{}".format(args.command[0])
+
+    denv = {}
+    if not args.clean:
         denv = builtins.__xonsh__.env.detype()
-        try:
-            os.execvpe(args[0], args, denv)
-        except FileNotFoundError as e:
-            return (
-                None,
-                "xonsh: exec: file not found: {}: {}" "\n".format(e.args[1], args[0]),
-                1,
-            )
+
+    try:
+        os.execvpe(command, args.command, denv)
+    except FileNotFoundError as e:
+        return (
+            None,
+            "xonsh: exec: file not found: {}: {}"
+            "\n".format(e.args[1], args.command[0]),
+            1,
+        )
 
 
 class AWitchAWitch(argparse.Action):
@@ -747,9 +780,9 @@ def make_default_aliases():
         "source-cmd": source_cmd,
         "source-foreign": source_foreign,
         "history": xhm.history_main,
-        "replay": replay_main,
         "trace": trace,
         "timeit": timeit_alias,
+        "umask": umask,
         "xonfig": xonfig,
         "scp-resume": ["rsync", "--partial", "-h", "--progress", "--rsh=ssh"],
         "showcmd": showcmd,
@@ -760,6 +793,8 @@ def make_default_aliases():
         "xpip": detect_xpip_alias(),
         "xonsh-reset": xonsh_reset,
     }
+    if ON_POSIX:
+        default_aliases["ulimit"] = ulimit
     if ON_WINDOWS:
         # Borrow builtin commands from cmd.exe.
         windows_cmd_aliases = {
