@@ -544,7 +544,14 @@ def complete_completer(prefix, line, start, end, ctx):
     args = line.split(" ")
     if len(args) == 0 or args[0] != "completer":
         return None
+
+    if end < len(line) and line[end] != " ":
+        # completing in a middle of a word
+        # (e.g. "completer some<TAB>thing")
+        return None
+
     curix = args.index(prefix)
+
     compnames = set(builtins.__xonsh__.completers.keys())
     if curix == 1:
         possible = {"list", "help", "add", "remove"}
@@ -581,12 +588,12 @@ def complete_completer(prefix, line, start, end, ctx):
 xl = _LazyModule.load('xonsh', 'xonsh.lazyasd', 'xl')
 @xl.lazyobject
 def PIP_RE():
-    return re.compile(r"\bx?pip(?:\d|\.)*")
+    return re.compile(r"\bx?pip(?:\d|\.)*\b")
 
 
 @xl.lazyobject
 def PIP_LIST_RE():
-    return re.compile(r"\bx?pip(?:\d|\.)* (?:uninstall|show)")
+    return re.compile(r"\bx?pip(?:\d|\.)*\b (?:uninstall|show)")
 
 
 @xl.lazyobject
@@ -662,6 +669,59 @@ def justify(s, max_length, left_pad=0):
     txt = textwrap.wrap(s, width=max_length, subsequent_indent=" " * left_pad)
     return "\n".join(txt)
 
+
+class RichCompletion(str):
+    """A rich completion that completers can return instead of a string
+
+    Parameters
+    ----------
+    value : str
+        The completion's actual value.
+    prefix_len : int
+        Length of the prefix to be replaced in the completion.
+        If None, the default prefix len will be used.
+    display : str
+        Text to display in completion option list.
+        If None, ``value`` will be used.
+    description : str
+        Extra text to display when the completion is selected.
+    """
+
+    def __new__(cls, value, prefix_len=None, display=None, description=""):
+        completion = super().__new__(cls, value)
+
+        completion.prefix_len = prefix_len
+        completion.display = display or value
+        completion.description = description
+
+        return completion
+
+    def __repr__(self):
+        return "RichCompletion({}, prefix_len={}, display={}, description={})".format(
+            repr(str(self)),
+            self.prefix_len,
+            repr(self.display),
+            repr(self.description),
+        )
+
+
+def get_ptk_completer():
+    """Get the current PromptToolkitCompleter
+
+    This is usefull for completers that want to use
+    PromptToolkitCompleter.current_document (the current multiline document).
+
+    Call this function lazily since in '.xonshrc' the shell doesn't exist.
+
+    Returns
+    -------
+    The PromptToolkitCompleter if running with ptk, else returns None
+    """
+    if __xonsh__.shell is None or __xonsh__.shell.shell_type != "prompt_toolkit":
+        return None
+
+    return __xonsh__.shell.shell.pt_completer
+
 #
 # xompletions
 #
@@ -1155,7 +1215,7 @@ def _add_cdpaths(paths, prefix):
             test_glob, ignore_case=(not csc), sort_result=glob_sorted
         ):
             if os.path.isdir(s):
-                paths.add(os.path.basename(s))
+                paths.add(os.path.relpath(s, cdp))
 
 
 def _quote_to_use(x):
@@ -1167,7 +1227,15 @@ def _quote_to_use(x):
         return single
 
 
-def _quote_paths(paths, start, end, append_end=True):
+def _is_directory_in_cdpath(path):
+    env = builtins.__xonsh__.env
+    for cdp in env.get("CDPATH"):
+        if os.path.isdir(os.path.join(cdp, path)):
+            return True
+    return False
+
+
+def _quote_paths(paths, start, end, append_end=True, cdpath=False):
     expand_path = builtins.__xonsh__.expand_path
     out = set()
     space = " "
@@ -1187,7 +1255,8 @@ def _quote_paths(paths, start, end, append_end=True):
         end = orig_end
         if start == "" and need_quotes:
             start = end = _quote_to_use(s)
-        if os.path.isdir(expand_path(s)):
+        expanded = expand_path(s)
+        if os.path.isdir(expanded) or (cdpath and _is_directory_in_cdpath(expanded)):
             _tail = slash
         elif end == "":
             _tail = space
@@ -1315,10 +1384,15 @@ def complete_path(prefix, line, start, end, ctx, cdpath=True, filtfunc=None):
         # e.g., ~/u/ro completes to ~/lou/carcolh
         # see above functions for details.
         p = _splitpath(os.path.expanduser(prefix))
-        if len(p) != 0:
-            if p[0] == "":
-                basedir = ("",)
-                p = p[1:]
+        p_len = len(p)
+        if p_len != 0:
+            relative_char = ["", ".", ".."]
+            if p[0] in relative_char:
+                i = 0
+                while i < p_len and p[i] in relative_char:
+                    i += 1
+                basedir = p[:i]
+                p = p[i:]
             else:
                 basedir = None
             matches_so_far = {basedir}
@@ -1341,7 +1415,7 @@ def complete_path(prefix, line, start, end, ctx, cdpath=True, filtfunc=None):
         _add_cdpaths(paths, prefix)
     paths = set(filter(filtfunc, paths))
     paths, _ = _quote_paths(
-        {_normpath(s) for s in paths}, path_str_start, path_str_end, append_end
+        {_normpath(s) for s in paths}, path_str_start, path_str_end, append_end, cdpath
     )
     paths.update(filter(filtfunc, _dots(prefix)))
     paths.update(filter(filtfunc, _env(prefix)))
@@ -1669,6 +1743,10 @@ def complete_base(prefix, line, start, end, ctx):
     and paths.  If we are completing the first argument, complete based on
     valid commands and python names.
     """
+    if line.strip() and prefix != line:
+        # don't do unnecessary completions
+        return set()
+
     # get and unpack python completions
     python_comps = complete_python(prefix, line, start, end, ctx)
     if isinstance(python_comps, cabc.Sequence):
