@@ -1866,13 +1866,11 @@ class Env(cabc.MutableMapping):
 
     def get_docs(self, key, default=None):
         """Gets the documentation for the environment variable."""
-        vd = self._vars.get(key, None)
+        vd = self._vars.get(key, default)
         if vd is None:
-            if default is None:
-                default = Var()
-            return default
+            vd = Var(default="", doc_default="")
         if vd.doc_default is DefaultNotGiven:
-            dval = pprint.pformat(self._vars.get(key, "<default not set>").default)
+            dval = pprint.pformat(vd.default)
             vd = vd._replace(doc_default=dval)
         return vd
 
