@@ -48,9 +48,9 @@ from xonsh.platform import (
     DEFAULT_ENCODING,
     ON_LINUX,
     ON_WINDOWS,
-    PYTHON_VERSION_INFO,
     expanduser,
     os_environ,
+    pygments_version_info,
 )
 
 
@@ -515,9 +515,9 @@ def _have_open_triple_quotes(s):
 
 
 def get_line_continuation():
-    """ The line continuation characters used in subproc mode. In interactive
-         mode on Windows the backslash must be preceded by a space. This is because
-         paths on Windows may end in a backslash.
+    """The line continuation characters used in subproc mode. In interactive
+    mode on Windows the backslash must be preceded by a space. This is because
+    paths on Windows may end in a backslash.
     """
     if (
         ON_WINDOWS
@@ -691,7 +691,7 @@ def indent(instr, nspaces=4, ntabs=0, flatten=False):
 
 
 def get_sep():
-    """ Returns the appropriate filepath separator char depending on OS and
+    """Returns the appropriate filepath separator char depending on OS and
     xonsh options set
     """
     if ON_WINDOWS and builtins.__xonsh__.env.get("FORCE_POSIX_PATHS"):
@@ -774,15 +774,6 @@ def _yield_accessible_unix_file_names(path):
 def _executables_in_posix(path):
     if not os.path.exists(path):
         return
-    elif PYTHON_VERSION_INFO < (3, 5, 0):
-        for fname in os.listdir(path):
-            fpath = os.path.join(path, fname)
-            if (
-                os.path.exists(fpath)
-                and os.access(fpath, os.X_OK)
-                and (not os.path.isdir(fpath))
-            ):
-                yield fname
     else:
         yield from _yield_accessible_unix_file_names(path)
 
@@ -791,14 +782,7 @@ def _executables_in_windows(path):
     if not os.path.isdir(path):
         return
     extensions = builtins.__xonsh__.env["PATHEXT"]
-    if PYTHON_VERSION_INFO < (3, 5, 0):
-        for fname in os.listdir(path):
-            fpath = os.path.join(path, fname)
-            if os.path.exists(fpath) and not os.path.isdir(fpath):
-                base_name, ext = os.path.splitext(fname)
-                if ext.upper() in extensions:
-                    yield fname
-    else:
+    try:
         for x in scandir(path):
             try:
                 is_file = x.is_file()
@@ -811,6 +795,11 @@ def _executables_in_windows(path):
             base_name, ext = os.path.splitext(fname)
             if ext.upper() in extensions:
                 yield fname
+    except FileNotFoundError:
+        # On Windows, there's no guarantee for the directory to really
+        # exist even if isdir returns True. This may happen for instance
+        # if the path contains trailing spaces.
+        return
 
 
 def executables_in(path):
@@ -825,23 +814,58 @@ def executables_in(path):
         return
 
 
-def command_not_found(cmd):
+def debian_command_not_found(cmd):
     """Uses the debian/ubuntu command-not-found utility to suggest packages for a
     command that cannot currently be found.
     """
     if not ON_LINUX:
         return ""
-    elif not os.path.isfile("/usr/lib/command-not-found"):
-        # utility is not on PATH
+
+    cnf = builtins.__xonsh__.commands_cache.lazyget(
+        "command-not-found", ("/usr/lib/command-not-found",)
+    )[0]
+
+    if not os.path.isfile(cnf):
         return ""
-    c = "/usr/lib/command-not-found {0}; exit 0"
+
+    c = "{0} {1}; exit 0"
     s = subprocess.check_output(
-        c.format(cmd), universal_newlines=True, stderr=subprocess.STDOUT, shell=True
+        c.format(cnf, cmd),
+        universal_newlines=True,
+        stderr=subprocess.STDOUT,
+        shell=True,
     )
     s = "\n".join(s.rstrip().splitlines()).strip()
     return s
 
 
+def conda_suggest_command_not_found(cmd, env):
+    """Uses conda-suggest to suggest packages for a command that cannot
+    currently be found.
+    """
+    try:
+        from conda_suggest import find
+    except ImportError:
+        return ""
+    return find.message_string(
+        cmd, conda_suggest_path=env.get("CONDA_SUGGEST_PATH", None)
+    )
+
+
+def command_not_found(cmd, env):
+    """Uses various mechanism to suggest packages for a command that cannot
+    currently be found.
+    """
+    if ON_LINUX:
+        rtn = debian_command_not_found(cmd)
+    else:
+        rtn = ""
+    conda = conda_suggest_command_not_found(cmd, env)
+    if conda:
+        rtn = rtn + "\n\n" + conda if rtn else conda
+    return rtn
+
+
 def suggest_commands(cmd, env, aliases):
     """Suggests alternative commands given an environment and aliases."""
     if not env.get("SUGGEST_COMMANDS"):
@@ -874,7 +898,7 @@ def suggest_commands(cmd, env, aliases):
     num = min(len(suggested), max_sugg)
 
     if num == 0:
-        rtn = command_not_found(cmd)
+        rtn = command_not_found(cmd, env)
     else:
         oneof = "" if num == 1 else "one of "
         tips = "Did you mean {}the following?".format(oneof)
@@ -884,7 +908,7 @@ def suggest_commands(cmd, env, aliases):
             "    {: <{}} {}".format(key + ":", length, val) for key, val in items
         )
         rtn = "{}\n{}".format(tips, alternatives)
-        c = command_not_found(cmd)
+        c = command_not_found(cmd, env)
         rtn += ("\n\n" + c) if len(c) > 0 else ""
     return rtn
 
@@ -1014,7 +1038,7 @@ def escape_windows_cmd_string(s):
 
 
 def argvquote(arg, force=False):
-    """ Returns an argument quoted in such a way that that CommandLineToArgvW
+    """Returns an argument quoted in such a way that that CommandLineToArgvW
     on Windows will return the argument string unchanged.
     This is the same thing Popen does when supplied with an list of arguments.
     Arguments in a command line should be separated by spaces; this
@@ -1174,6 +1198,11 @@ def is_bool(x):
     return isinstance(x, bool)
 
 
+def is_bool_or_none(x):
+    """Tests if something is a boolean or None."""
+    return (x is None) or isinstance(x, bool)
+
+
 def is_logfile_opt(x):
     """
     Checks if x is a valid $XONSH_TRACEBACK_LOGFILE option. Returns False
@@ -1193,10 +1222,7 @@ def to_logfile_opt(x):
     the filepath if it is a writable file or None if the filepath is not
     valid, informing the user on stderr about the invalid choice.
     """
-    superclass = pathlib.PurePath
-    if PYTHON_VERSION_INFO >= (3, 6, 0):
-        superclass = os.PathLike
-    if isinstance(x, superclass):
+    if isinstance(x, os.PathLike):
         x = str(x)
     if is_logfile_opt(x):
         return x
@@ -1240,6 +1266,20 @@ def to_bool(x):
         return bool(x)
 
 
+def to_bool_or_none(x):
+    """"Converts to a boolean or none in a semantically meaningful way."""
+    if x is None or isinstance(x, bool):
+        return x
+    elif isinstance(x, str):
+        low_x = x.lower()
+        if low_x == "none":
+            return None
+        else:
+            return False if x.lower() in _FALSES else True
+    else:
+        return bool(x)
+
+
 def to_itself(x):
     """No conversion, returns itself."""
     return x
@@ -1252,6 +1292,14 @@ def bool_to_str(x):
     return "1" if x else ""
 
 
+def bool_or_none_to_str(x):
+    """Converts a bool or None value to a string."""
+    if x is None:
+        return "None"
+    else:
+        return "1" if x else ""
+
+
 _BREAKS = LazyObject(
     lambda: frozenset(["b", "break", "s", "skip", "q", "quit"]), globals(), "_BREAKS"
 )
@@ -1460,8 +1508,8 @@ def bool_seq_to_csv(x):
 
 
 def ptk2_color_depth_setter(x):
-    """ Setter function for $PROMPT_TOOLKIT_COLOR_DEPTH. Also
-        updates os.environ so prompt toolkit can pickup the value.
+    """Setter function for $PROMPT_TOOLKIT_COLOR_DEPTH. Also
+    updates os.environ so prompt toolkit can pickup the value.
     """
     x = str(x)
     if x in {
@@ -1547,21 +1595,6 @@ def dict_to_str(x):
     return str(x)
 
 
-def setup_win_unicode_console(enable):
-    """"Enables or disables unicode display on windows."""
-    try:
-        import win_unicode_console
-    except ImportError:
-        win_unicode_console = False
-    enable = to_bool(enable)
-    if ON_WINDOWS and win_unicode_console:
-        if enable:
-            win_unicode_console.enable()
-        else:
-            win_unicode_console.disable()
-    return enable
-
-
 # history validation
 
 _min_to_sec = lambda x: 60.0 * float(x)
@@ -1654,7 +1687,7 @@ def is_history_backend(x):
 
 
 def is_dynamic_cwd_width(x):
-    """ Determine if the input is a valid input for the DYNAMIC_CWD_WIDTH
+    """Determine if the input is a valid input for the DYNAMIC_CWD_WIDTH
     environment variable.
     """
     return (
@@ -1866,7 +1899,7 @@ WIN_BOLD_COLOR_MAP = LazyObject(_win_bold_color_map, globals(), "WIN_BOLD_COLOR_
 
 def hardcode_colors_for_win10(style_map):
     """Replace all ansi colors with hardcoded colors to avoid unreadable defaults
-       in conhost.exe
+    in conhost.exe
     """
     modified_style = {}
     if not builtins.__xonsh__.env["PROMPT_TOOLKIT_COLOR_DEPTH"]:
@@ -1892,8 +1925,9 @@ def hardcode_colors_for_win10(style_map):
 
 
 def ansicolors_to_ptk1_names(stylemap):
-    """Converts ansicolor names in a stylemap to old PTK1 color names
-    """
+    """Converts ansicolor names in a stylemap to old PTK1 color names"""
+    if pygments_version_info() and pygments_version_info() >= (2, 4, 0):
+        return stylemap
     modified_stylemap = {}
     for token, style_str in stylemap.items():
         for color, ptk1_color in ANSICOLOR_NAMES_MAP.items():
@@ -1905,7 +1939,7 @@ def ansicolors_to_ptk1_names(stylemap):
 
 def intensify_colors_for_cmd_exe(style_map):
     """Returns a modified style to where colors that maps to dark
-       colors are replaced with brighter versions.
+    colors are replaced with brighter versions.
     """
     modified_style = {}
     replace_colors = {
@@ -1929,9 +1963,12 @@ def intensify_colors_on_win_setter(enable):
     environment variable.
     """
     enable = to_bool(enable)
-    if builtins.__xonsh__.shell is not None:
-        if hasattr(builtins.__xonsh__.shell.shell.styler, "style_name"):
-            delattr(builtins.__xonsh__.shell.shell.styler, "style_name")
+    if (
+        hasattr(builtins.__xonsh__, "shell")
+        and builtins.__xonsh__.shell is not None
+        and hasattr(builtins.__xonsh__.shell.shell.styler, "style_name")
+    ):
+        delattr(builtins.__xonsh__.shell.shell.styler, "style_name")
     return enable
 
 
@@ -1942,16 +1979,27 @@ def format_std_prepost(template, env=None):
     if not template:
         return ""
     env = builtins.__xonsh__.env if env is None else env
-    shell = builtins.__xonsh__.shell.shell
-    try:
-        s = shell.prompt_formatter(template)
-    except Exception:
-        print_exception()
-    # \001\002 is there to fool pygments into not returning an empty string
-    # for potentially empty input. This happens when the template is just a
-    # color code with no visible text.
     invis = "\001\002"
-    s = shell.format_color(invis + s + invis, force_string=True)
+    if builtins.__xonsh__.shell is None:
+        # shell hasn't fully started up (probably still in xonshrc)
+        from xonsh.prompt.base import PromptFormatter
+        from xonsh.ansi_colors import ansi_partial_color_format
+
+        pf = PromptFormatter()
+        s = pf(template)
+        style = env.get("XONSH_COLOR_STYLE")
+        s = ansi_partial_color_format(invis + s + invis, hide=False, style=style)
+    else:
+        # shell has fully started. do the normal thing
+        shell = builtins.__xonsh__.shell.shell
+        try:
+            s = shell.prompt_formatter(template)
+        except Exception:
+            print_exception()
+        # \001\002 is there to fool pygments into not returning an empty string
+        # for potentially empty input. This happens when the template is just a
+        # color code with no visible text.
+        s = shell.format_color(invis + s + invis, force_string=True)
     s = s.replace(invis, "")
     return s
 
@@ -2130,9 +2178,9 @@ def expandvars(path):
         for match in POSIX_ENVVAR_REGEX.finditer(path):
             name = match.group("envvar")
             if name in env:
-                ensurer = env.get_ensurer(name)
+                detyper = env.get_detyper(name)
                 val = env[name]
-                value = str(val) if ensurer.detype is None else ensurer.detype(val)
+                value = str(val) if detyper is None else detyper(val)
                 value = str(val) if value is None else value
                 path = POSIX_ENVVAR_REGEX.sub(value, path, count=1)
     return path
@@ -2242,37 +2290,21 @@ def _iglobpath(s, ignore_case=False, sort_result=None, include_dotfiles=None):
         include_dotfiles = builtins.__xonsh__.env.get("DOTGLOB")
     if ignore_case:
         s = expand_case_matching(s)
-    if sys.version_info > (3, 5):
-        if "**" in s and "**/*" not in s:
-            s = s.replace("**", "**/*")
-        if include_dotfiles:
-            dotted_s, dotmodified = _dotglobstr(s)
-        # `recursive` is only a 3.5+ kwarg.
-        if sort_result:
-            paths = glob.glob(s, recursive=True)
-            if include_dotfiles and dotmodified:
-                paths.extend(glob.iglob(dotted_s, recursive=True))
-            paths.sort()
-            paths = iter(paths)
-        else:
-            paths = glob.iglob(s, recursive=True)
-            if include_dotfiles and dotmodified:
-                paths = itertools.chain(glob.iglob(dotted_s, recursive=True), paths)
-        return paths, s
+    if "**" in s and "**/*" not in s:
+        s = s.replace("**", "**/*")
+    if include_dotfiles:
+        dotted_s, dotmodified = _dotglobstr(s)
+    if sort_result:
+        paths = glob.glob(s, recursive=True)
+        if include_dotfiles and dotmodified:
+            paths.extend(glob.iglob(dotted_s, recursive=True))
+        paths.sort()
+        paths = iter(paths)
     else:
-        if include_dotfiles:
-            dotted_s, dotmodified = _dotglobstr(s)
-        if sort_result:
-            paths = glob.glob(s)
-            if include_dotfiles and dotmodified:
-                paths.extend(glob.iglob(dotted_s))
-            paths.sort()
-            paths = iter(paths)
-        else:
-            paths = glob.iglob(s)
-            if include_dotfiles and dotmodified:
-                paths = itertools.chain(glob.iglob(dotted_s), paths)
-        return paths, s
+        paths = glob.iglob(s, recursive=True)
+        if include_dotfiles and dotmodified:
+            paths = itertools.chain(glob.iglob(dotted_s, recursive=True), paths)
+    return paths, s
 
 
 def iglobpath(s, ignore_case=False, sort_result=None, include_dotfiles=None):
