@@ -2,11 +2,8 @@
 import shlex
 import sys
 import re
+import builtins
 
-from prompt_toolkit.keys import Keys
-from prompt_toolkit.filters import Condition, EmacsInsertMode, ViInsertMode
-
-from xonsh.platform import ptk_shell_type
 
 __all__ = ()
 
@@ -40,26 +37,7 @@ def bash_preproc(cmd, **kw):
                 print("xonsh: no previous commands match '!{}'".format(arg))
                 return ""
 
-    return re.sub(r"!([!$^*]|[\w]+)", replace_bang, cmd.strip())
-
-
-@events.on_ptk_create
-def custom_keybindings(bindings, **kw):
-    if ptk_shell_type() == "prompt_toolkit2":
-        handler = bindings.add
-    else:
-        handler = bindings.registry.add_binding
-
-    insert_mode = ViInsertMode() | EmacsInsertMode()
-
-    @Condition
-    def last_command_exists():
-        return len(__xonsh__.history) > 0
-
-    @handler(Keys.Escape, ".", filter=last_command_exists & insert_mode)
-    def recall_last_arg(event):
-        arg = __xonsh__.history[-1].cmd.split()[-1]
-        event.current_buffer.insert_text(arg)
+    return re.sub(r"!([!$^*]|[\w]+)", replace_bang, cmd)
 
 
 def alias(args, stdin=None):
@@ -85,3 +63,4 @@ def alias(args, stdin=None):
 
 
 aliases["alias"] = alias
+builtins.__xonsh__.env["THREAD_SUBPROCS"] = False
